__all__ = [
    "__title__", "__summary__", "__uri__", "__version__", "__author__",
    "__email__", "__license__", "__copyright__",
]

__title__ = "munch-core"
__summary__ = "Munch is an emailing platform."
__uri__ = "https://github.com/crunchmail/munch-core"

__version__ = "0.1.0"

__author__ = u"Crunchmail"
__email__ = "dev@crunchmail.com"

__license__ = "GNU AGPLv3"
__copyright__ = "Copyright 2016 %s" % __author__
