# -*- coding: utf-8 -*-
# Generated by Django 1.9.3 on 2016-05-27 08:15
from __future__ import unicode_literals

import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('campaigns', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='AbuseNotification',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date', models.DateTimeField(auto_now=True, verbose_name='date')),
                ('first_name', models.CharField(max_length=60, verbose_name='firstname')),
                ('second_name', models.CharField(max_length=60, verbose_name='lastname')),
                ('organization', models.CharField(blank=True, max_length=100, verbose_name='organization')),
                ('contact_phone', models.CharField(blank=True, max_length=50, validators=[django.core.validators.RegexValidator(message='Phone number format must be like "+33123456789"', regex='^\\+?1?[\\d\\.\\s]{9,20}$')], verbose_name='phone number')),
                ('contact_mail', models.EmailField(blank=True, max_length=254, verbose_name='email')),
                ('comment', models.TextField(blank=True, verbose_name='additional information')),
                ('mail', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='campaigns.Mail', verbose_name='contact email report')),
            ],
            options={
                'verbose_name_plural': 'abuse reports',
                'verbose_name': 'abuse report',
            },
        ),
    ]
