# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-09-23 17:53
from __future__ import unicode_literals

from django.db import migrations, models
import munch.apps.domains.fields
import munch.core.utils.validators


class Migration(migrations.Migration):

    dependencies = [
        ('domains', '0002_permissions'),
    ]

    operations = [
        migrations.AlterField(
            model_name='sendingdomain',
            name='app_domain',
            field=models.CharField(blank=True, default='', help_text='Domain to use for email links generation', max_length=200, validators=[munch.core.utils.validators.DomainNameValidator()], verbose_name='Custom application domain'),
        ),
        migrations.AlterField(
            model_name='sendingdomain',
            name='app_domain_status',
            field=munch.apps.domains.fields.DomainCheckField(choices=[('ok', 'Configured'), ('ko', 'Not configured'), ('bad', 'Badly configured'), ('pending', 'Checking'), ('unknown', 'Unknown')], default='unknown', max_length=10, verbose_name='Custom application domain status'),
        ),
        migrations.AlterField(
            model_name='sendingdomain',
            name='app_domain_status_date',
            field=models.DateTimeField(blank=True, null=True, verbose_name='Custom application domain status last change'),
        ),
    ]
