# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-09-23 13:16
from __future__ import unicode_literals

from django.db import migrations


def comments_default(apps, schema_editor):
    AbuseNotification = apps.get_model('abuse', 'AbuseNotification')
    for report in AbuseNotification.objects.all():
        if not report.comments:
            report.comments = 'null'
            report.save()


class Migration(migrations.Migration):

    dependencies = [
        ('abuse', '0002_auto_20160923_1516'),
    ]

    operations = [
        migrations.RunPython(comments_default)
    ]
