# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-09-28 12:15
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('users', '0005_auto_20160926_1810'),
    ]

    operations = [
        migrations.AddField(
            model_name='munchuser',
            name='invited_by',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL, verbose_name='invited by'),
        ),
    ]
