# -*- coding: utf-8 -*-
# Generated by Django 1.9.3 on 2016-05-27 08:26
from __future__ import unicode_literals

import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import munch.apps.upload_store.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('users', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Image',
            fields=[
                ('upload_date', models.DateTimeField(auto_now=True)),
                ('expiration', models.DurationField()),
                ('hash', models.CharField(max_length=64, primary_key=True, serialize=False)),
                ('file', models.ImageField(upload_to=munch.apps.upload_store.models.Upload.get_path)),
                ('width', models.PositiveSmallIntegerField(default=600, validators=[django.core.validators.MaxValueValidator(600)])),
                ('organization', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='images', to='users.Organization')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]
