<!DOCTYPE html>
<html>
	<head>
        <meta charset="utf-8" />
		<title>test_form</title>
	</head>
	<body>
        <form action="{% url 'subscription' uuid=uuid %}" method="post">
            {% csrf_token %}
            {{ form }}
            <input type="submit" />
        </form>
	</body>
</html>
