# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-09-24 17:41
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('users', '0003_auto_20160924_1312'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='organization',
            name='password_reset_link',
        ),
    ]
