#content {
    padding-top: 10%
}

h2 {
    margin-bottom: 30px;
}

.top_alert {
    position: fixed;
    top: 0;
    left: 0;
    width: 100%;
}

.framed {
    border: 2px solid #eee;
    border-radius: 4px;
    padding: 48px;
}
