# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-09-23 13:16
from __future__ import unicode_literals

import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('abuse', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='abusenotification',
            name='comment',
        ),
        migrations.RemoveField(
            model_name='abusenotification',
            name='contact_mail',
        ),
        migrations.RemoveField(
            model_name='abusenotification',
            name='contact_phone',
        ),
        migrations.RemoveField(
            model_name='abusenotification',
            name='first_name',
        ),
        migrations.RemoveField(
            model_name='abusenotification',
            name='organization',
        ),
        migrations.RemoveField(
            model_name='abusenotification',
            name='second_name',
        ),
        migrations.AddField(
            model_name='abusenotification',
            name='comments',
            field=models.TextField(null=True, verbose_name='comments'),
        ),
        migrations.AddField(
            model_name='abusenotification',
            name='contact_email',
            field=models.CharField(blank=True, max_length=50, validators=[django.core.validators.EmailValidator()], verbose_name='contact email'),
        ),
        migrations.AddField(
            model_name='abusenotification',
            name='contact_name',
            field=models.CharField(blank=True, max_length=100, verbose_name='contact name'),
        ),
        migrations.AlterField(
            model_name='abusenotification',
            name='mail',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='campaigns.Mail', verbose_name='mail'),
        ),
    ]
