.animated {
  -webkit-animation-duration: 1s;
  animation-duration: 1s;
  -webkit-animation-fill-mode: both;
  animation-fill-mode: both; }

@-webkit-keyframes slideUp {
  to {
    height: 0;
    padding-top: 0;
    padding-bottom: 0; } }

@keyframes slideUp {
  to {
    height: 0;
    padding-top: 0;
    padding-bottom: 0; } }

.slideUp {
  overflow: hidden;
  -webkit-animation-name: slideUp;
  animation-name: slideUp; }

@-webkit-keyframes slideDown {
  from {
    height: 0;
    padding-top: 0;
    padding-bottom: 0; } }

@keyframes slideDown {
  from {
    height: 0;
    padding-top: 0;
    padding-bottom: 0; } }

.slideDown {
  overflow: hidden;
  -webkit-animation-name: slideDown;
  animation-name: slideDown; }

@-webkit-keyframes slideInRight {
  from {
    -webkit-transform: translate3d(100%, 0, 0);
    transform: translate3d(100%, 0, 0);
    visibility: visible; }
  to {
    -webkit-transform: translate3d(0, 0, 0);
    transform: translate3d(0, 0, 0); } }

@keyframes slideInRight {
  from {
    -webkit-transform: translate3d(100%, 0, 0);
    transform: translate3d(100%, 0, 0);
    visibility: visible; }
  to {
    -webkit-transform: translate3d(0, 0, 0);
    transform: translate3d(0, 0, 0); } }

.slideInRight {
  -webkit-animation-name: slideInRight;
  animation-name: slideInRight; }

@-webkit-keyframes slideInLeft {
  from {
    -webkit-transform: translate3d(-100%, 0, 0);
    transform: translate3d(-100%, 0, 0);
    visibility: visible; }
  to {
    -webkit-transform: translate3d(0, 0, 0);
    transform: translate3d(0, 0, 0); } }

@keyframes slideInLeft {
  from {
    -webkit-transform: translate3d(-100%, 0, 0);
    transform: translate3d(-100%, 0, 0);
    visibility: visible; }
  to {
    -webkit-transform: translate3d(0, 0, 0);
    transform: translate3d(0, 0, 0); } }

.slideInLeft {
  -webkit-animation-name: slideInLeft;
  animation-name: slideInLeft; }

@-webkit-keyframes slideInDown {
  from {
    -webkit-transform: translate3d(0, -100%, 0);
    transform: translate3d(0, -100%, 0);
    visibility: visible; }
  to {
    -webkit-transform: translate3d(0, 0, 0);
    transform: translate3d(0, 0, 0); } }

@keyframes slideInDown {
  from {
    -webkit-transform: translate3d(0, -100%, 0);
    transform: translate3d(0, -100%, 0);
    visibility: visible; }
  to {
    -webkit-transform: translate3d(0, 0, 0);
    transform: translate3d(0, 0, 0); } }

.slideInDown {
  -webkit-animation-name: slideInDown;
  animation-name: slideInDown; }

@-webkit-keyframes slideOutLeft {
  from {
    -webkit-transform: translate3d(0, 0, 0);
    transform: translate3d(0, 0, 0); }
  to {
    visibility: hidden;
    -webkit-transform: translate3d(-100%, 0, 0);
    transform: translate3d(-100%, 0, 0); } }

@keyframes slideOutLeft {
  from {
    -webkit-transform: translate3d(0, 0, 0);
    transform: translate3d(0, 0, 0); }
  to {
    visibility: hidden;
    -webkit-transform: translate3d(-100%, 0, 0);
    transform: translate3d(-100%, 0, 0); } }

.slideOutLeft {
  -webkit-animation-name: slideOutLeft;
  animation-name: slideOutLeft; }

@-webkit-keyframes slideOutRight {
  from {
    -webkit-transform: translate3d(0, 0, 0);
    transform: translate3d(0, 0, 0); }
  to {
    visibility: hidden;
    -webkit-transform: translate3d(100%, 0, 0);
    transform: translate3d(100%, 0, 0); } }

@keyframes slideOutRight {
  from {
    -webkit-transform: translate3d(0, 0, 0);
    transform: translate3d(0, 0, 0); }
  to {
    visibility: hidden;
    -webkit-transform: translate3d(100%, 0, 0);
    transform: translate3d(100%, 0, 0); } }

.slideOutRight {
  -webkit-animation-name: slideOutRight;
  animation-name: slideOutRight; }

@-webkit-keyframes slideOutUp {
  from {
    -webkit-transform: translate3d(0, 0, 0);
    transform: translate3d(0, 0, 0); }
  to {
    visibility: hidden;
    -webkit-transform: translate3d(0, -100%, 0);
    transform: translate3d(0, -100%, 0); } }

@keyframes slideOutUp {
  from {
    -webkit-transform: translate3d(0, 0, 0);
    transform: translate3d(0, 0, 0); }
  to {
    visibility: hidden;
    -webkit-transform: translate3d(0, -100%, 0);
    transform: translate3d(0, -100%, 0); } }

.slideOutUp {
  -webkit-animation-name: slideOutUp;
  animation-name: slideOutUp; }

@-webkit-keyframes fadeIn {
  from {
    opacity: 0; }
  to {
    opacity: 1; } }

@keyframes fadeIn {
  from {
    opacity: 0; }
  to {
    opacity: 1; } }

.fadeIn {
  -webkit-animation-name: fadeIn;
  animation-name: fadeIn; }

@-webkit-keyframes fadeOut {
  from {
    opacity: 1; }
  to {
    opacity: 0; } }

@keyframes fadeOut {
  from {
    opacity: 1; }
  to {
    opacity: 0; } }

.fadeOut {
  -webkit-animation-name: fadeOut;
  animation-name: fadeOut; }

@-webkit-keyframes zoomIn {
  from {
    opacity: 0;
    -webkit-transform: scale3d(0.3, 0.3, 0.3);
    transform: scale3d(0.3, 0.3, 0.3); }
  50% {
    opacity: 1; } }

@keyframes zoomIn {
  from {
    opacity: 0;
    -webkit-transform: scale3d(0.3, 0.3, 0.3);
    transform: scale3d(0.3, 0.3, 0.3); }
  50% {
    opacity: 1; } }

.zoomIn {
  -webkit-animation-name: zoomIn;
  animation-name: zoomIn; }

@-webkit-keyframes zoomOut {
  from {
    opacity: 1; }
  50% {
    opacity: 0;
    -webkit-transform: scale3d(0.3, 0.3, 0.3);
    transform: scale3d(0.3, 0.3, 0.3); }
  to {
    opacity: 0; } }

@keyframes zoomOut {
  from {
    opacity: 1; }
  50% {
    opacity: 0;
    -webkit-transform: scale3d(0.3, 0.3, 0.3);
    transform: scale3d(0.3, 0.3, 0.3); }
  to {
    opacity: 0; } }

.zoomOut {
  -webkit-animation-name: zoomOut;
  animation-name: zoomOut; }

@-webkit-keyframes pulse {
  from {
    -webkit-transform: scale3d(1, 1, 1);
    transform: scale3d(1, 1, 1); }
  50% {
    -webkit-transform: scale3d(1.03, 1.03, 1.03);
    transform: scale3d(1.03, 1.03, 1.03); }
  to {
    -webkit-transform: scale3d(1, 1, 1);
    transform: scale3d(1, 1, 1); } }

@keyframes pulse {
  from {
    -webkit-transform: scale3d(1, 1, 1);
    transform: scale3d(1, 1, 1); }
  50% {
    -webkit-transform: scale3d(1.03, 1.03, 1.03);
    transform: scale3d(1.03, 1.03, 1.03); }
  to {
    -webkit-transform: scale3d(1, 1, 1);
    transform: scale3d(1, 1, 1); } }

.pulse {
  -webkit-animation-name: pulse;
  animation-name: pulse; }

@-webkit-keyframes shake {
  from, to {
    -webkit-transform: translate3d(0, 0, 0);
    transform: translate3d(0, 0, 0); }
  10%, 30%, 50%, 70%, 90% {
    -webkit-transform: translate3d(-7px, 0, 0);
    transform: translate3d(-7px, 0, 0); }
  20%, 40%, 60%, 80% {
    -webkit-transform: translate3d(7px, 0, 0);
    transform: translate3d(7px, 0, 0); } }

@keyframes shake {
  from, to {
    -webkit-transform: translate3d(0, 0, 0);
    transform: translate3d(0, 0, 0); }
  10%, 30%, 50%, 70%, 90% {
    -webkit-transform: translate3d(-7px, 0, 0);
    transform: translate3d(-7px, 0, 0); }
  20%, 40%, 60%, 80% {
    -webkit-transform: translate3d(7px, 0, 0);
    transform: translate3d(7px, 0, 0); } }

.shake {
  -webkit-animation-name: shake;
  animation-name: shake; }

@-webkit-keyframes bounce {
  from, 20%, 53%, 80%, to {
    -webkit-animation-timing-function: cubic-bezier(0.215, 0.61, 0.355, 1);
    animation-timing-function: cubic-bezier(0.215, 0.61, 0.355, 1);
    -webkit-transform: translate3d(0, 0, 0);
    transform: translate3d(0, 0, 0); }
  40%, 43% {
    -webkit-animation-timing-function: cubic-bezier(0.755, 0.05, 0.855, 0.06);
    animation-timing-function: cubic-bezier(0.755, 0.05, 0.855, 0.06);
    -webkit-transform: translate3d(0, -20px, 0);
    transform: translate3d(0, -20px, 0); }
  70% {
    -webkit-animation-timing-function: cubic-bezier(0.755, 0.05, 0.855, 0.06);
    animation-timing-function: cubic-bezier(0.755, 0.05, 0.855, 0.06);
    -webkit-transform: translate3d(0, -15px, 0);
    transform: translate3d(0, -15px, 0); }
  90% {
    -webkit-transform: translate3d(0, -4px, 0);
    transform: translate3d(0, -4px, 0); } }

@keyframes bounce {
  from, 20%, 53%, 80%, to {
    -webkit-animation-timing-function: cubic-bezier(0.215, 0.61, 0.355, 1);
    animation-timing-function: cubic-bezier(0.215, 0.61, 0.355, 1);
    -webkit-transform: translate3d(0, 0, 0);
    transform: translate3d(0, 0, 0); }
  40%, 43% {
    -webkit-animation-timing-function: cubic-bezier(0.755, 0.05, 0.855, 0.06);
    animation-timing-function: cubic-bezier(0.755, 0.05, 0.855, 0.06);
    -webkit-transform: translate3d(0, -20px, 0);
    transform: translate3d(0, -20px, 0); }
  70% {
    -webkit-animation-timing-function: cubic-bezier(0.755, 0.05, 0.855, 0.06);
    animation-timing-function: cubic-bezier(0.755, 0.05, 0.855, 0.06);
    -webkit-transform: translate3d(0, -15px, 0);
    transform: translate3d(0, -15px, 0); }
  90% {
    -webkit-transform: translate3d(0, -4px, 0);
    transform: translate3d(0, -4px, 0); } }

.bounce {
  -webkit-animation-name: bounce;
  animation-name: bounce;
  -webkit-transform-origin: center bottom;
  transform-origin: center bottom; }

@-webkit-keyframes rotate {
  from {
    -webkit-transform: rotate(0deg);
    transform: rotate(0deg); }
  to {
    -webkit-transform: rotate(360deg);
    transform: rotate(360deg); } }

@keyframes rotate {
  from {
    -webkit-transform: rotate(0deg);
    transform: rotate(0deg); }
  to {
    -webkit-transform: rotate(360deg);
    transform: rotate(360deg); } }

.rotate {
  -webkit-animation-name: rotate;
  animation-name: rotate; }

html {
  box-sizing: border-box; }

*,
*:before,
*:after {
  box-sizing: inherit; }

* {
  margin: 0;
  padding: 0;
  border: 0;
  outline: 0;
  font-size: 100%;
  vertical-align: baseline;
  background: transparent; }

a:active,
a:hover {
  outline: 0; }

::-moz-focus-inner {
  border: 0;
  padding: 0; }

img,
video,
audio {
  max-width: 100%; }

img,
video {
  height: auto; }

input[type="radio"],
input[type="checkbox"] {
  cursor: pointer;
  font-size: 110%;
  position: relative;
  margin-right: 3px;
  padding: 0; }

input[type="search"] {
  -webkit-appearance: textfield; }

input[type="search"]::-webkit-search-decoration,
input[type="search"]::-webkit-search-cancel-button {
  -webkit-appearance: none; }

body {
  font-family: "Helvetica Neue", Helvetica, sans-serif;
  line-height: 24px;
  font-size: 16px;
  color: #333;
  background-color: #fff; }

a {
  color: #255bbd; }

a:hover {
  color: #f34248; }

h1.title, h1, h2, h3, h4, h5, h6 {
  font-family: "Helvetica Neue", Helvetica, sans-serif;
  font-weight: bold;
  text-rendering: optimizeLegibility;
  color: #000;
  margin-bottom: 12px; }

h1.title {
  font-size: 60px;
  line-height: 64px; }

h1,
.h1 {
  font-size: 48px;
  line-height: 52px; }

h2,
.h2 {
  font-size: 36px;
  line-height: 40px; }

h3,
.h3 {
  font-size: 24px;
  line-height: 32px; }

h4,
.h4 {
  font-size: 18px;
  line-height: 24px; }

h5,
.h5 {
  font-size: 16px;
  line-height: 24px; }

h6,
.h6 {
  font-size: 12px;
  line-height: 20px;
  text-transform: uppercase;
  letter-spacing: 0.05em; }

p + h2,
p + h3,
p + h4,
p + h5,
p + h6,
ul + h2,
ul + h3,
ul + h4,
ul + h5,
ul + h6,
ol + h2,
ol + h3,
ol + h4,
ol + h5,
ol + h6,
dl + h2,
dl + h3,
dl + h4,
dl + h5,
dl + h6,
blockquote + h2,
blockquote + h3,
blockquote + h4,
blockquote + h5,
blockquote + h6,
hr + h2,
hr + h3,
hr + h4,
hr + h5,
hr + h6,
pre + h2,
pre + h3,
pre + h4,
pre + h5,
pre + h6,
table + h2,
table + h3,
table + h4,
table + h5,
table + h6,
form + h2,
form + h3,
form + h4,
form + h5,
form + h6,
figure + h2,
figure + h3,
figure + h4,
figure + h5,
figure + h6 {
  margin-top: 32px; }

.subheading {
  margin-top: -8px;
  margin-bottom: 8px;
  font-weight: 300; }

ul, ol,
ul ul,
ol ol,
ul ol,
ol ul {
  margin: 0 0 0 24px; }

ol ol li {
  list-style-type: lower-alpha; }

ol ol ol li {
  list-style-type: lower-roman; }

nav ul, nav ol {
  margin: 0;
  list-style: none; }

dl dt {
  font-weight: 700; }

dd {
  margin-left: 24px; }

p, blockquote, hr, pre, ol, ul, dl, table, fieldset, figure, address, form {
  margin-bottom: 16px; }

hr {
  border-bottom: 1px solid rgba(0, 0, 0, 0.1);
  margin-top: -1px; }

blockquote {
  padding-left: 24px;
  border-left: 2px solid rgba(0, 0, 0, 0.125);
  font-style: italic;
  color: rgba(0, 0, 0, 0.65); }

pre, code, samp, var, kbd {
  font-family: Consolas, Monaco, "Courier New", monospace;
  font-size: 14px; }

pre {
  color: rgba(0, 0, 0, 0.85);
  background: #f8f8f8;
  padding: 20px;
  border-radius: 4px;
  line-height: 20px;
  overflow: none;
  white-space: pre-wrap; }

.small, time, cite, small, figcaption {
  font-size: 12px;
  line-height: 16px; }

cite {
  color: rgba(0, 0, 0, 0.5); }

figcaption {
  font-style: italic;
  padding-top: 4px;
  padding-bottom: 4px; }

figure figcaption {
  position: relative;
  top: -4px; }

abbr[title], dfn[title] {
  border-bottom: 1px dotted rgba(0, 0, 0, 0.5);
  cursor: help; }

mark, code, samp, kbd {
  padding: 4px 8px 2px 8px;
  display: inline-block;
  line-height: 1;
  border-radius: 4px;
  color: rgba(0, 0, 0, 0.85); }

pre code {
  font-size: 100%;
  white-space: pre;
  padding: 0;
  background: none;
  line-height: 24px; }

mark {
  background-color: #fed443; }

code {
  background: #f4f4f4; }

var {
  color: rgba(0, 0, 0, 0.6);
  font-style: normal; }

strong, b {
  line-height: 1; }

samp {
  color: #fff;
  background: #5a89df; }

kbd {
  border: 1px solid rgba(0, 0, 0, 0.1); }

sub,
sup {
  font-size: x-small;
  line-height: 0;
  margin-left: 4px;
  position: relative; }

sup {
  top: -4px; }

sub {
  bottom: -2px; }

.row {
  display: -ms-flexbox;
  display: -webkit-flex;
  display: flex;
  -ms-flex-wrap: wrap;
  -webkit-flex-wrap: wrap;
  flex-wrap: wrap;
  margin-left: -2.12766%; }
  .row.centered {
    -ms-flex-pack: center;
    -webkit-justify-content: center;
    justify-content: center; }
    .row.centered .col {
      padding-left: 1.06383%;
      padding-right: 1.06383%; }
  .row.right {
    -ms-flex-pack: end;
    -webkit-justify-content: flex-end;
    justify-content: flex-end; }
  .row.bottom {
    -ms-flex-align: end;
    -webkit-align-items: flex-end;
    align-items: flex-end; }
  .row.around {
    -ms-flex-pack: distribute;
    -webkit-justify-content: space-around;
    justify-content: space-around; }
  .row.between {
    -ms-flex-pack: justify;
    -webkit-justify-content: space-between;
    justify-content: space-between; }
  .row.equal .col {
    display: -ms-flexbox;
    display: -webkit-flex;
    display: flex; }
    .row.equal .col > div {
      -ms-flex-grow: 1;
      -webkit-flex-grow: 1;
      flex-grow: 1; }
  .row.splice {
    margin-left: 0; }
    .row.splice .col {
      padding-left: 0; }

.col {
  -ms-flex-grow: 1;
  -webkit-flex-grow: 1;
  flex-grow: 1;
  -webkit-flex-basis: 0;
  -ms-flex-basis: 0;
  flex-basis: 0;
  max-width: 100%;
  padding-left: 2.12766%; }
  .col.first {
    order: -1; }
  .col.last {
    order: 1; }

@media (max-width: 768px) {
  .col.first-on-small {
    order: -1; }
  .col.last-on-small {
    order: 1; } }

.pull-right {
  margin-left: auto; }

.col-1 {
  flex-basis: 8.33333%;
  max-width: 8.33333%;
  width: 8.33333%; }

.col-2 {
  flex-basis: 16.66667%;
  max-width: 16.66667%;
  width: 16.66667%; }

.col-3 {
  flex-basis: 25%;
  max-width: 25%;
  width: 25%; }

.col-4 {
  flex-basis: 33.33333%;
  max-width: 33.33333%;
  width: 33.33333%; }

.col-5 {
  flex-basis: 41.66667%;
  max-width: 41.66667%;
  width: 41.66667%; }

.col-6 {
  flex-basis: 50%;
  max-width: 50%;
  width: 50%; }

.col-7 {
  flex-basis: 58.33333%;
  max-width: 58.33333%;
  width: 58.33333%; }

.col-8 {
  flex-basis: 66.66667%;
  max-width: 66.66667%;
  width: 66.66667%; }

.col-9 {
  flex-basis: 75%;
  max-width: 75%;
  width: 75%; }

.col-10 {
  flex-basis: 83.33333%;
  max-width: 83.33333%;
  width: 83.33333%; }

.col-11 {
  flex-basis: 91.66667%;
  max-width: 91.66667%;
  width: 91.66667%; }

.col-12 {
  flex-basis: 100%;
  max-width: 100%;
  width: 100%; }

.offset-1 {
  margin-left: 8.33333%; }

.offset-2 {
  margin-left: 16.66667%; }

.offset-3 {
  margin-left: 25%; }

.offset-4 {
  margin-left: 33.33333%; }

.offset-5 {
  margin-left: 41.66667%; }

.offset-6 {
  margin-left: 50%; }

.offset-7 {
  margin-left: 58.33333%; }

.offset-8 {
  margin-left: 66.66667%; }

.offset-9 {
  margin-left: 75%; }

.offset-10 {
  margin-left: 83.33333%; }

.offset-11 {
  margin-left: 91.66667%; }

.row-layout {
  display: -ms-flexbox;
  display: -webkit-flex;
  display: flex;
  -ms-flex-wrap: nowrap;
  -webkit-flex-wrap: nowrap;
  flex-wrap: nowrap;
  -ms-flex-pack: center;
  -webkit-justify-content: center;
  justify-content: center; }

.col-fluid {
  -ms-flex-grow: 1;
  -webkit-flex-grow: 1;
  flex-grow: 1;
  width: 100%; }

@media (max-width: 768px) {
  .row-layout {
    display: block; }
  .col-fixed {
    -ms-flex: 1 !important;
    -webkit-flex: 1 !important;
    flex: 1 !important; }
  .row-layout > .col-fixed {
    order: 1; }
  .row-layout > .col-fluid {
    order: 2; } }

@media (max-width: 768px) {
  .row {
    margin-left: 0; }
  .col:not(th):not(td),
  [class^="col-"]:not(th):not(td),
  [class*=" col-"]:not(th):not(td) {
    flex-basis: 100%;
    max-width: 100%;
    width: 100%; }
  .col {
    -ms-flex-direction: column;
    -webkit-flex-direction: column;
    flex-direction: column;
    padding-left: 0; }
  .pull-right,
  [class^="offset-"],
  [class*=" offset-"] {
    margin-left: 0; } }

table {
  border-collapse: collapse;
  border-spacing: 0;
  max-width: 100%;
  width: 100%;
  empty-cells: show;
  font-size: 15px; }

th {
  text-align: left;
  font-weight: 700;
  vertical-align: bottom; }

td {
  vertical-align: top; }

tr.align-middle td,
td.align-middle {
  vertical-align: middle; }

th,
td {
  padding: 16px;
  padding-bottom: 15px;
  border-bottom: 1px solid #eee; }
  th:first-child,
  td:first-child {
    padding-left: 0; }
  th:last-child,
  td:last-child {
    padding-right: 0; }

tfoot th,
tfoot td {
  color: rgba(0, 0, 0, 0.5); }

table.bordered {
  margin-top: -1px; }
  table.bordered td,
  table.bordered th {
    border: 1px solid #eee; }

table.striped tr:nth-child(odd) td {
  background: #f4f4f4; }

table.bordered td:first-child,
table.bordered th:first-child,
table.striped td:first-child,
table.striped th:first-child {
  padding-left: 16px; }

table.bordered td:last-child,
table.bordered th:last-child,
table.striped td:last-child,
table.striped th:last-child {
  padding-right: 16px; }

table.flat td,
table.flat th {
  border: none;
  padding: 0; }

input,
select,
textarea,
fieldset {
  font-family: "Helvetica Neue", Helvetica, sans-serif;
  font-size: 14px; }

input[type="email"],
input[type="number"],
input[type="search"],
input[type="text"],
input[type="date"],
input[type="tel"],
input[type="url"],
input[type="password"] {
  height: 40px;
  padding: 8px 10px;
  vertical-align: middle;
  -webkit-appearance: none;
  background: #fff;
  border: 1px solid #dfdfdf;
  border-radius: 2px; }

select,
textarea {
  display: block;
  width: 100%;
  line-height: 1.75;
  padding: 8px 10px;
  border-radius: 2px;
  background: #fff;
  border: 1px solid #dfdfdf; }

select:not([multiple]) {
  cursor: pointer;
  height: 40px;
  vertical-align: middle; }

label {
  vertical-align: middle; }

textarea:focus,
select:focus,
input:focus {
  outline: none;
  background: #fff;
  border-color: #bbb; }

textarea:disabled,
select:disabled,
input:disabled,
textarea.disabled,
select.disabled,
input.disabled {
  resize: none;
  opacity: 0.75;
  cursor: default;
  font-style: italic;
  color: rgba(0, 0, 0, 0.5); }

fieldset {
  padding: 39px 32px;
  margin-bottom: 24px;
  border: 1px solid rgba(0, 0, 0, 0.1);
  border-radius: 4px; }

legend {
  font-weight: bold;
  font-size: 12px;
  text-transform: uppercase;
  padding: 0 1em;
  margin-left: -1em;
  top: 2px;
  position: relative;
  line-height: 0; }

.form {
  margin-bottom: 0; }
  .form .form-item {
    margin-bottom: 24px; }
  .form input[type="email"],
  .form input[type="number"],
  .form input[type="search"],
  .form input[type="text"],
  .form input[type="date"],
  .form input[type="tel"],
  .form input[type="url"],
  .form input[type="password"],
  .form select,
  .form textarea {
    display: block;
    width: 100%; }
  .form input.width-50,
  .form textarea.width-50,
  .form select.width-50 {
    width: 50%; }
  .form label {
    font-size: 14px;
    color: #111;
    font-weight: normal;
    display: block; }
  .form .checkboxes label,
  .form label.checkbox {
    text-transform: none;
    font-weight: normal;
    cursor: pointer;
    color: inherit;
    font-size: 15px; }
  .form .checkboxes label {
    margin-bottom: 8px; }
    .form .checkboxes label:last-child {
      margin-bottom: 0; }
  .form .checkboxes-inline {
    display: block; }
    .form .checkboxes-inline label {
      font-size: inherit;
      color: inherit;
      display: inline-block;
      margin-right: 16px;
      text-transform: none;
      font-weight: normal; }
  .form span.error,
  .form div.error,
  .form span.success,
  .form div.success {
    text-transform: none;
    font-weight: normal;
    font-size: 12px; }
  .form span.error,
  .form span.success {
    margin-left: .4rem; }

.form-inline textarea,
.form-inline select,
.form-inline input[type="email"],
.form-inline input[type="number"],
.form-inline input[type="search"],
.form-inline input[type="text"],
.form-inline input[type="date"],
.form-inline input[type="tel"],
.form-inline input[type="url"],
.form-inline input[type="password"] {
  width: auto;
  display: inline-block; }

input.big,
select.big {
  font-size: 18px;
  height: 48px; }

input.big {
  padding: 16px 10px; }

select.big {
  padding: 6px 10px; }

.desc {
  text-transform: none;
  margin-top: 4px;
  color: rgba(0, 0, 0, 0.5);
  font-size: 12px;
  line-height: 16px;
  font-weight: normal; }

ul.desc {
  margin-bottom: 8px; }

span.desc {
  margin-left: 4px;
  line-height: 20px; }

input.error,
textarea.error,
select.error {
  margin-top: -1px;
  background: none;
  background-color: rgba(243, 66, 72, 0.1);
  border: 1px solid #f34248; }

input.success,
textarea.success,
select.success {
  margin-top: -1px;
  background: none;
  background-color: rgba(96, 174, 77, 0.1);
  border: 1px solid #60ae4d; }

.controls {
  display: -ms-flexbox;
  display: -webkit-flex;
  display: flex; }
  .controls input {
    -ms-flex: 1;
    -webkit-flex: 1;
    flex: 1;
    margin-bottom: 0 !important; }
  .controls .button,
  .controls span {
    -ms-flex-shrink: 0;
    -webkit-flex-shrink: 0;
    flex-shrink: 0; }
  .controls span {
    font-weight: normal;
    background-color: #eee;
    padding: 0 12px;
    color: #555;
    font-size: 12px;
    line-height: 40px;
    white-space: nowrap; }
  .controls .button {
    border-radius: 0 4px 4px 0; }

@media (max-width: 768px) {
  .form input,
  .form textarea,
  .form select {
    font-size: 16px; }
    .form input.width-50,
    .form textarea.width-50,
    .form select.width-50 {
      width: 100%; } }

button,
.button {
  font-family: "Helvetica Neue", Helvetica, sans-serif;
  font-size: 14px;
  font-weight: normal;
  text-decoration: none;
  cursor: pointer;
  display: inline-block;
  line-height: 1.125rem;
  min-height: 40px;
  padding: 10px 28px;
  border: 1px solid transparent;
  vertical-align: middle;
  -webkit-appearance: none;
  color: #000;
  background-color: #eee;
  border-radius: 4px; }

input.button {
  height: auto; }

button:hover,
.button:hover {
  outline: none;
  color: rgba(0, 0, 0, 0.6);
  text-decoration: none;
  box-shadow: inset 0 0 0 40px rgba(0, 0, 0, 0.125); }

.button:disabled,
.button.disabled {
  cursor: default;
  font-style: normal;
  opacity: .5; }
  .button:disabled:hover,
  .button.disabled:hover {
    color: #000;
    box-shadow: none; }

.button.outline {
  background: none;
  font-weight: normal; }

.button.outline:hover {
  color: rgba(0, 0, 0, 0.6);
  border-color: rgba(0, 0, 0, 0.2);
  box-shadow: none; }

.button.outline,
.button.outline:disabled:hover,
.button.outline.disabled:hover {
  color: rgba(0, 0, 0, 0.8);
  border-color: rgba(0, 0, 0, 0.4); }

.button.small {
  font-size: 12px;
  min-height: 32px;
  padding: 6px 20px; }
  .button.small.upper {
    font-size: 10px; }

.button.big {
  font-size: 18px;
  min-height: 48px;
  padding: 14px 48px; }
  .button.big.upper {
    font-size: 13px; }

.button.upper {
  font-size: 11px; }

.button.primary {
  background: #3c74d9;
  color: #fff; }
  .button.primary:focus, .button.primary:hover {
    color: rgba(255, 255, 255, 0.6); }
    .button.primary:focus:disabled, .button.primary:focus.disabled, .button.primary:hover:disabled, .button.primary:hover.disabled {
      color: #fff;
      background: #3c74d9; }
  .button.primary.outline:hover {
    color: rgba(60, 116, 217, 0.6);
    border-color: rgba(60, 116, 217, 0.4); }
  .button.primary.outline, .button.primary.outline:disabled:hover, .button.primary.outline.disabled:hover {
    color: #3c74d9;
    border-color: #3c74d9;
    background: none; }

.button.inverted {
  background: #fff;
  color: #000; }
  .button.inverted:focus, .button.inverted:hover {
    color: rgba(0, 0, 0, 0.6); }
    .button.inverted:focus:disabled, .button.inverted:focus.disabled, .button.inverted:hover:disabled, .button.inverted:hover.disabled {
      color: #000;
      background: #fff; }
  .button.inverted.outline:hover {
    color: rgba(255, 255, 255, 0.6);
    border-color: rgba(255, 255, 255, 0.4); }
  .button.inverted.outline, .button.inverted.outline:disabled:hover, .button.inverted.outline.disabled:hover {
    color: white;
    border-color: white;
    background: none; }

.buttons.group {
  display: inline-block;
  vertical-align: bottom; }
  .buttons.group .button {
    float: left;
    margin-left: -1px;
    border-radius: 0; }
    .buttons.group .button:first-child {
      border-radius: 4px 0 0 4px; }
    .buttons.group .button:last-child {
      border-radius: 0 4px 4px 0; }

.label {
  font-size: 10px;
  font-weight: normal;
  display: inline;
  line-height: 1;
  top: -1px;
  text-align: center;
  text-transform: uppercase;
  text-decoration: none;
  border-radius: 4px;
  position: relative;
  vertical-align: middle;
  color: #000;
  background: #f4f4f4;
  padding: 2px 8px;
  letter-spacing: .01em; }
  .label a,
  .label a:hover {
    color: #000;
    text-decoration: none; }

.label.primary {
  background: #3c74d9; }
  .label.primary,
  .label.primary a {
    color: #fff; }
  .label.primary.outline {
    border-color: #3c74d9; }
    .label.primary.outline,
    .label.primary.outline a {
      color: #3c74d9; }
  .label.primary.tag {
    background: none; }
    .label.primary.tag,
    .label.primary.tag a {
      color: #3c74d9; }

.label.error {
  background: #f34248; }
  .label.error,
  .label.error a {
    color: #fff; }
  .label.error.outline {
    border-color: #f34248; }
    .label.error.outline,
    .label.error.outline a {
      color: #f34248; }
  .label.error.tag {
    background: none; }
    .label.error.tag,
    .label.error.tag a {
      color: #f34248; }

.label.success {
  background: #60ae4d; }
  .label.success,
  .label.success a {
    color: #fff; }
  .label.success.outline {
    border-color: #60ae4d; }
    .label.success.outline,
    .label.success.outline a {
      color: #60ae4d; }
  .label.success.tag {
    background: none; }
    .label.success.tag,
    .label.success.tag a {
      color: #60ae4d; }

.label.warning {
  background: #fed443; }
  .label.warning,
  .label.warning a {
    color: #000; }
  .label.warning.outline {
    border-color: #fed443; }
    .label.warning.outline,
    .label.warning.outline a {
      color: #fed443; }
  .label.warning.tag {
    background: none; }
    .label.warning.tag,
    .label.warning.tag a {
      color: #fed443; }

.label.inverted {
  background: #fff; }
  .label.inverted,
  .label.inverted a {
    color: #000; }
  .label.inverted.outline {
    border-color: #fff; }
    .label.inverted.outline,
    .label.inverted.outline a {
      color: #fff; }
  .label.inverted.tag {
    background: none; }
    .label.inverted.tag,
    .label.inverted.tag a {
      color: #fff; }

.label.outline {
  background: none;
  padding: 1px 7px;
  border: 1px solid rgba(0, 0, 0, 0.2); }
  .label.outline,
  .label.outline a {
    color: rgba(0, 0, 0, 0.65); }

.badge {
  padding: 2px 6px;
  border-radius: 24px; }

.badge.outline {
  padding: 1px 5px; }

.label.tag {
  padding: 0;
  background: none;
  font-weight: bold;
  letter-spacing: .02em; }

.label.tag.primary,
.label.tag.primary a {
  color: #3c74d9; }

.label.tag.error,
.label.tag.error a {
  color: #f34248; }

.label.tag.success,
.label.tag.success a {
  color: #60ae4d; }

.label.tag.warning,
.label.tag.warning a {
  color: #fed443; }

.label.tag.inverted,
.label.tag.inverted a {
  color: #fff; }

.button:hover .badge {
  opacity: .6; }

h6 .label {
  top: -3px; }

.alert {
  position: relative;
  padding: 15px 24px;
  padding-right: 40px;
  line-height: 20px;
  font-size: 14px;
  color: #000;
  background: #f4f4f4;
  border: 1px solid #dbdbdb;
  display: block;
  font-weight: normal;
  border-radius: 4px;
  margin-bottom: 24px; }
  .alert a {
    color: #000;
    font-weight: bold; }
    .alert a:hover {
      -moz-transition: all linear 0.2s;
      transition: all linear 0.2s;
      color: rgba(0, 0, 0, 0.6); }
  .alert h5 {
    margin-bottom: 4px; }
  .alert p,
  .alert ul,
  .alert ol {
    margin-bottom: 0; }
  .alert .close {
    position: absolute;
    top: 15px;
    right: 8px; }

.alert.primary {
  color: #3c74d9;
  background-color: rgba(60, 116, 217, 0.15);
  border: 1px solid rgba(60, 116, 217, 0.15); }
  .alert.primary a,
  .alert.primary h5 {
    color: #3c74d9; }
  .alert.primary a:hover {
    color: #000; }

.alert.error {
  color: #f34248;
  background-color: rgba(243, 66, 72, 0.15);
  border: 1px solid rgba(243, 66, 72, 0.15); }
  .alert.error a,
  .alert.error h5 {
    color: #f34248; }
  .alert.error a:hover {
    color: #000; }

.alert.success {
  color: #60ae4d;
  background-color: rgba(96, 174, 77, 0.15);
  border: 1px solid rgba(96, 174, 77, 0.15); }
  .alert.success a,
  .alert.success h5 {
    color: #60ae4d; }
  .alert.success a:hover {
    color: #000; }

.alert.warning {
  color: #c09501;
  background-color: rgba(254, 212, 67, 0.15);
  border: 1px solid rgba(254, 212, 67, 0.15);
  border-color: rgba(254, 212, 67, 0.5); }
  .alert.warning a,
  .alert.warning h5 {
    color: #c09501; }
  .alert.warning a:hover {
    color: #000; }

.message {
  display: none;
  position: fixed;
  border-radius: 4px;
  z-index: 500;
  top: 16px;
  right: 16px;
  width: 320px;
  line-height: 20px;
  font-size: 14px;
  padding: 16px 20px;
  color: rgba(0, 0, 0, 0.9);
  background: #f4f4f4; }
  .message .close {
    position: absolute;
    top: 6px;
    right: 6px; }
  .message h1,
  .message h2,
  .message h3,
  .message h4,
  .message h5,
  .message h6 {
    color: inherit; }

.message ul,
.message ol {
  margin-bottom: 0; }

.message.line {
  top: 0;
  right: 0;
  left: 0;
  width: 100%;
  max-width: none;
  border-radius: 0;
  padding-right: 56px; }
  .message.line .close {
    top: 16px;
    right: 16px; }

.message .buttons {
  margin-top: 16px; }

.message .buttons a {
  display: inline-block;
  text-transform: uppercase;
  font-size: 12px;
  margin-right: 8px; }

.message.line .buttons {
  margin-top: 0;
  display: inline-block; }

.message.primary {
  background: #3c74d9;
  color: rgba(255, 255, 255, 0.9); }

.message.error {
  background: #f34248;
  color: rgba(255, 255, 255, 0.9); }

.message.success {
  background: #60ae4d;
  color: rgba(255, 255, 255, 0.9); }

.message.warning {
  background: #fed443; }

.message.primary a,
.message.error a,
.message.success a {
  color: rgba(255, 255, 255, 0.9); }
  .message.primary a:hover,
  .message.error a:hover,
  .message.success a:hover {
    color: rgba(255, 255, 255, 0.7); }

.tabs {
  margin-bottom: 24px;
  font-size: 14px; }

.tabs li em,
.tabs li.active a {
  color: #000;
  border: 1px solid rgba(0, 0, 0, 0.1);
  cursor: default;
  text-decoration: none;
  background: none; }

.tabs em,
.tabs a {
  position: relative;
  top: 1px;
  font-style: normal;
  display: block;
  padding: 7px 24px;
  border: 1px solid transparent;
  color: rgba(0, 0, 0, 0.5);
  text-decoration: none;
  border-top-right-radius: 4px;
  border-top-left-radius: 4px; }

.tabs a:hover {
  -moz-transition: all linear 0.2s;
  transition: all linear 0.2s;
  color: #000;
  text-decoration: underline;
  background-color: #f2f2f2; }

@media (min-width: 768px) {
  .tabs ul {
    display: -ms-flexbox;
    display: -webkit-flex;
    display: flex;
    margin-top: -1px;
    border-bottom: 1px solid rgba(0, 0, 0, 0.1); }
  .tabs li em,
  .tabs li.active a {
    border-bottom: 1px solid #fff; } }

.breadcrumb {
  font-size: 12px;
  margin-bottom: 24px; }

.breadcrumb ul {
  display: -ms-flexbox;
  display: -webkit-flex;
  display: flex;
  -webkit-align-items: center;
  -ms-align-items: center;
  align-items: center; }

.breadcrumb.centered ul {
  -ms-flex-pack: center;
  -webkit-justify-content: center;
  justify-content: center; }

.breadcrumb em,
.breadcrumb a {
  font-style: normal;
  padding: 0 10px;
  display: inline-block;
  white-space: nowrap; }

.breadcrumb em {
  color: rgba(0, 0, 0, 0.5); }

.breadcrumb a {
  color: #000;
  text-decoration: none; }
  .breadcrumb a:hover {
    -moz-transition: all linear 0.2s;
    transition: all linear 0.2s;
    color: rgba(0, 0, 0, 0.6);
    text-decoration: underline; }

.breadcrumb li:after {
  display: inline-block;
  content: '/';
  color: rgba(0, 0, 0, 0.4); }

.breadcrumb li:last-child:after {
  display: none; }

.breadcrumb li:first-child em,
.breadcrumb li:first-child a {
  padding-left: 0; }

.breadcrumb li.active em,
.breadcrumb li.active a {
  text-decoration: none;
  cursor: text;
  color: rgba(0, 0, 0, 0.4); }

.pagination {
  margin: 24px 0;
  font-size: 14px; }

.pagination ul {
  display: -ms-flexbox;
  display: -webkit-flex;
  display: flex; }

.pagination.centered ul {
  -ms-flex-pack: center;
  -webkit-justify-content: center;
  justify-content: center; }

.pagination em,
.pagination a {
  display: inline-block;
  font-style: normal;
  padding: 8px 16px;
  line-height: 1;
  border-radius: 4px;
  white-space: nowrap;
  border: 1px solid transparent; }

.pagination a {
  text-decoration: none;
  color: #000; }
  .pagination a:hover {
    color: rgba(0, 0, 0, 0.5);
    background: #f4f4f4; }

.pagination em,
.pagination li.active a {
  color: rgba(0, 0, 0, 0.5);
  background: #f4f4f4;
  cursor: text; }

.pagination.upper {
  font-size: 11px; }

.pager em,
.pager a {
  border-radius: 24px;
  border-color: rgba(0, 0, 0, 0.1); }

.pager li {
  -webkit-flex-basis: 50%;
  -ms-flex-basis: 50%;
  flex-basis: 50%; }

.pager li.next {
  text-align: right; }

.pager.centered li {
  -webkit-flex-basis: auto;
  -ms-flex-basis: auto;
  flex-basis: auto;
  margin-left: 4px;
  margin-right: 4px; }

.pager.flat em,
.pager.flat a {
  border: none;
  display: block;
  padding: 0; }

.pager.flat a {
  font-weight: bold; }
  .pager.flat a:hover {
    background: none;
    text-decoration: underline; }

@media (max-width: 768px) {
  .pager.flat ul {
    -ms-flex-direction: column;
    -webkit-flex-direction: column;
    flex-direction: column; }
  .pager.flat li {
    -webkit-flex-basis: 100%;
    -ms-flex-basis: 100%;
    flex-basis: 100%;
    margin-bottom: 8px;
    text-align: left; } }

.dropdown {
  display: none;
  position: absolute;
  z-index: 100;
  top: 0;
  right: 0;
  width: 280px;
  color: #000;
  font-size: 14px;
  background: #fff;
  box-shadow: 0 10px 25px rgba(0, 0, 0, 0.15);
  border-radius: 8px;
  max-height: 300px;
  margin: 0;
  padding: 0;
  overflow: hidden; }

.dropdown.dropdown-mobile {
  position: fixed;
  top: 0;
  left: 0;
  right: 0;
  bottom: 0;
  width: 100%;
  max-height: none;
  border: none; }

.dropdown .close {
  margin: 20px auto; }

.dropdown.open {
  overflow: auto; }

.dropdown ul {
  list-style: none;
  margin: 0; }
  .dropdown ul li {
    border-bottom: 1px solid rgba(0, 0, 0, 0.07); }
    .dropdown ul li:last-child {
      border-bottom: none; }
  .dropdown ul a {
    display: block;
    padding: 8px 8px 7px 8px;
    text-decoration: none;
    color: #000; }
    .dropdown ul a:hover {
      background: rgba(0, 0, 0, 0.05); }

.modal-box {
  position: fixed;
  top: 0;
  left: 0;
  bottom: 0;
  right: 0;
  overflow-x: hidden;
  overflow-y: auto;
  z-index: 200; }

.modal {
  position: relative;
  margin: auto;
  margin-top: 16px;
  padding: 0;
  background: #fff;
  box-shadow: 0 10px 25px rgba(0, 0, 0, 0.15);
  border-radius: 8px;
  color: #000; }
  @media (max-width: 768px) {
    .modal input[type="text"],
    .modal input[type="password"],
    .modal input[type="email"],
    .modal input[type="url"],
    .modal textarea {
      font-size: 16px; } }

.modal-header {
  border-top-right-radius: 4px;
  border-top-left-radius: 4px;
  padding: 24px 32px;
  font-size: 18px;
  font-weight: bold;
  border-bottom: 1px solid rgba(0, 0, 0, 0.05); }

.modal-header:empty {
  display: none; }

.modal-body {
  padding: 36px 56px; }

.modal .close {
  line-height: 32.4px;
  height: 36px;
  width: 36px;
  font-size: 36px;
  position: absolute;
  top: 18px;
  right: 16px;
  opacity: .3; }
  .modal .close:hover {
    opacity: 1; }

@media (max-width: 768px) {
  .modal-header,
  .modal-body {
    padding: 24px; } }

.group:after {
  content: "";
  display: table;
  clear: both; }

.end {
  margin-bottom: 0 !important; }

.centered {
  margin-left: auto;
  margin-right: auto; }

.float-right {
  float: right; }
  @media (max-width: 768px) {
    .float-right {
      float: none; } }

.float-left {
  float: left; }
  @media (max-width: 768px) {
    .float-left {
      float: none; } }

.text-right {
  text-align: right; }

.text-center,
.text-centered {
  text-align: center; }

.hide {
  display: none; }

@media (max-width: 768px) {
  .hide-on-small {
    display: none !important; } }

.show-on-small {
  display: none !important; }
  @media (max-width: 768px) {
    .show-on-small {
      display: block !important; } }

@media print {
  .hide-on-print {
    display: none !important; }
  .show-on-print {
    display: block !important; } }

.invisible {
  visibility: hidden; }

.visible {
  visibility: visible; }

.black {
  color: #000; }

.muted {
  color: rgba(0, 0, 0, 0.5) !important; }

.soft-muted {
  color: rgba(0, 0, 0, 0.7) !important; }

a.muted:hover,
a.soft-muted:hover {
  color: #000 !important; }

.error {
  color: #f34248; }

.success {
  color: #60ae4d; }

.upper {
  text-transform: uppercase;
  letter-spacing: .08em; }

.normal {
  font-weight: normal !important; }

.light {
  font-weight: 300 !important; }

.italic {
  font-style: italic !important; }

.bold {
  font-weight: bold !important; }

.highlight {
  background-color: rgba(60, 116, 217, 0.075); }

.round {
  border-radius: 56px; }

input.round {
  border-radius: 24px; }

.nowrap,
.nowrap td {
  white-space: nowrap; }

.req {
  font-weight: bold;
  color: #f34248;
  font-size: 1.1em;
  text-transform: none; }

.close {
  font-weight: normal;
  text-align: center;
  display: inline-block;
  cursor: pointer;
  color: #000;
  text-decoration: none;
  opacity: 0.5;
  border: none;
  line-height: 25.2px;
  height: 28px;
  width: 28px;
  font-size: 28px; }
  .close:before {
    content: '\00D7'; }
  .close:hover {
    opacity: 1;
    text-decoration: none !important; }
  .close.inverted {
    color: #fff; }

.close.small {
  line-height: 18px;
  height: 20px;
  width: 20px;
  font-size: 20px; }

.close.big {
  line-height: 32.4px;
  height: 36px;
  width: 36px;
  font-size: 36px; }

@media (min-width: 768px) {
  .fullwidth {
    -ms-flex: auto;
    -webkit-flex: auto;
    flex: auto; }
  .fullwidth ul {
    display: -ms-flexbox;
    display: -webkit-flex;
    display: flex; }
  .fullwidth li {
    -ms-flex: auto;
    -webkit-flex: auto;
    flex: auto;
    text-align: center; }
    .fullwidth li em,
    .fullwidth li a {
      display: block; } }

.width-100 {
  width: 100%; }

.width-50 {
  width: 50%; }

@media (max-width: 768px) {
  .width-50 {
    width: 100% !important; } }

.fixed {
  position: fixed;
  top: 0;
  left: 0;
  z-index: 100;
  width: 100%; }

.no-scroll {
  overflow: hidden;
  position: fixed;
  top: 0;
  left: 0;
  width: 100%;
  height: 100% !important; }

.overlay {
  position: fixed;
  z-index: 200;
  top: 0;
  left: 0;
  right: 0;
  bottom: 0;
  background-color: rgba(255, 255, 255, 0.9); }

.scrollbar-measure {
  position: absolute;
  top: -9999px;
  width: 50px;
  height: 50px;
  overflow: scroll; }

.search-icon {
  font-family: Arial, Helvetica, sans-serif;
  display: inline-block;
  font-size: 18px;
  width: 16px;
  line-height: 1;
  font-style: normal;
  color: #000;
  position: relative;
  top: -1px;
  font-weight: bold;
  -webkit-transform: rotate(-45deg);
  -moz-transform: rotate(-45deg);
  -o-transform: rotate(-45deg); }
  .search-icon:before {
    content: "\26B2"; }
  .search-icon.inverted {
    color: #fff; }

button .search-icon {
  top: 0; }

button:hover .search-icon {
  opacity: .6; }

.menu-icon {
  font-family: Arial, Helvetica, sans-serif;
  display: inline-block;
  font-size: 15px;
  width: 16px;
  line-height: 1;
  font-style: normal;
  color: #000;
  font-weight: normal; }
  .menu-icon:before {
    content: "\2630"; }
  .menu-icon.inverted {
    color: #fff; }

button:hover .menu-icon {
  opacity: .6; }

.caret {
  position: relative;
  display: inline-block;
  height: 0;
  vertical-align: middle;
  width: 0;
  margin-left: 2px; }

.caret:before {
  content: '';
  position: absolute;
  top: -2px;
  left: 0;
  border-left: 4px solid transparent;
  border-right: 4px solid transparent; }

.caret.down:before {
  border-top: 4px solid #000; }

.caret.up:before {
  border-bottom: 4px solid #000; }

.caret.down.inverted:before {
  border-top-color: #fff; }

.caret.up.inverted:before {
  border-bottom-color: #fff; }

.button:hover .caret {
  opacity: .6; }

.offcanvas {
  background: #fff;
  position: fixed;
  padding: 24px;
  height: 100%;
  top: 0;
  left: 0;
  z-index: 300;
  overflow-y: scroll; }

.offcanvas .close {
  position: absolute;
  top: 8px;
  right: 8px; }

.offcanvas-left {
  border-right: 1px solid rgba(0, 0, 0, 0.1); }

.offcanvas-right {
  left: auto;
  right: 0;
  border-left: 1px solid rgba(0, 0, 0, 0.1); }

.offcanvas-push-body {
  position: relative; }

@media print {
  * {
    background: transparent !important;
    color: black !important;
    box-shadow: none !important;
    text-shadow: none !important; }
  a,
  a:visited {
    text-decoration: underline; }
  pre, blockquote {
    border: 1px solid #999;
    page-break-inside: avoid; }
  p, h2, h3 {
    orphans: 3;
    widows: 3; }
  thead {
    display: table-header-group; }
  tr, img {
    page-break-inside: avoid; }
  img {
    max-width: 100% !important; }
  h2, h3, h4 {
    page-break-after: avoid; }
  @page {
    margin: 0.5cm; } }
