{% load i18n %}
<div class="alert top_alert warning text-center">
    {% trans 'You have followed a link from a preview message.' %}<br />
    <strong>{% trans 'Your actions will not be saved.' %}</strong>
</div>
