# -*- coding: utf-8 -*-
# Generated by Django 1.9.3 on 2016-06-02 09:13
from __future__ import unicode_literals

import django.contrib.postgres.fields.hstore
from django.db import migrations, models
from django.contrib.postgres.operations import HStoreExtension


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        HStoreExtension(),
        migrations.CreateModel(
            name='LinkMap',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('identifier', models.CharField(db_index=True, max_length=50, unique=True, verbose_name='identifier')),
                ('link', models.URLField(verbose_name='link', max_length=500)),
            ],
        ),
        migrations.CreateModel(
            name='TrackRecord',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('identifier', models.CharField(db_index=True, max_length=150, verbose_name='identifier')),
                ('kind', models.CharField(db_index=True, max_length=50, verbose_name='kind')),
                ('properties', django.contrib.postgres.fields.hstore.HStoreField(blank=True, null=True, verbose_name='properties')),
                ('creation_date', models.DateTimeField(auto_now_add=True, verbose_name='creation date')),
            ],
            options={
                'ordering': ['creation_date'],
            },
        ),
    ]
