# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-09-26 16:10
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('users', '0004_remove_organization_password_reset_link'),
    ]

    operations = [
        migrations.AlterField(
            model_name='munchuser',
            name='first_name',
            field=models.CharField(max_length=50, verbose_name='first name'),
        ),
        migrations.AlterField(
            model_name='munchuser',
            name='last_name',
            field=models.CharField(max_length=50, verbose_name='last name'),
        ),
    ]
